@@ -303,14 +303,14 @@ binascii.b2a_uu
     data: Py_buffer
     /
     *
-    backtick: bool(accept={int}) = False
+    backtick: bool = False
 
 Uuencode line of data.
 [clinic start generated code]*/
 
 static PyObject *
 binascii_b2a_uu_impl(PyObject *module, Py_buffer *data, int backtick)
-/*[clinic end generated code: output=b1b99de62d9bbeb8 input=b26bc8d32b6ed2f6]*/
+/*[clinic end generated code: output=b1b99de62d9bbeb8 input=beb27822241095cd]*/
 {
     unsigned char *ascii_data;
     const unsigned char *bin_data;
@@ -375,7 +375,7 @@ binascii.a2b_base64
     data: ascii_buffer
     /
     *
-    strict_mode: bool(accept={int}) = False
+    strict_mode: bool = False
 
 Decode a line of base64 data.
 
@@ -386,7 +386,7 @@ Decode a line of base64 data.
 
 static PyObject *
 binascii_a2b_base64_impl(PyObject *module, Py_buffer *data, int strict_mode)
-/*[clinic end generated code: output=5409557788d4f975 input=3a30c4e3528317c6]*/
+/*[clinic end generated code: output=5409557788d4f975 input=c0c15fd0f8f9a62d]*/
 {
     assert(data->len >= 0);
 
@@ -424,6 +424,13 @@ binascii_a2b_base64_impl(PyObject *module, Py_buffer *data, int strict_mode)
         if (this_ch == BASE64_PAD) {
             padding_started = 1;
 
+            if (strict_mode && quad_pos == 0) {
+                state = get_binascii_state(module);
+                if (state) {
+                    PyErr_SetString(state->Error, "Excess padding not allowed");
+                }
+                goto error_end;
+            }
             if (quad_pos >= 2 && quad_pos + ++pads >= 4) {
                 /* A pad sequence means we should not parse more input.
                 ** We've already interpreted the data from the quad at this point.
@@ -521,14 +528,14 @@ binascii.b2a_base64
     data: Py_buffer
     /
     *
-    newline: bool(accept={int}) = True
+    newline: bool = True
 
 Base64-code line of data.
 [clinic start generated code]*/
 
 static PyObject *
 binascii_b2a_base64_impl(PyObject *module, Py_buffer *data, int newline)
-/*[clinic end generated code: output=4ad62c8e8485d3b3 input=6083dac5777fa45d]*/
+/*[clinic end generated code: output=4ad62c8e8485d3b3 input=0e20ff59c5f2e3e1]*/
 {
     unsigned char *ascii_data;
     const unsigned char *bin_data;
@@ -960,14 +967,14 @@ binascii_unhexlify_impl(PyObject *module, Py_buffer *hexstr)
 binascii.a2b_qp
 
     data: ascii_buffer
-    header: bool(accept={int}) = False
+    header: bool = False
 
 Decode a string of qp-encoded data.
 [clinic start generated code]*/
 
 static PyObject *
 binascii_a2b_qp_impl(PyObject *module, Py_buffer *data, int header)
-/*[clinic end generated code: output=e99f7846cfb9bc53 input=bf6766fea76cce8f]*/
+/*[clinic end generated code: output=e99f7846cfb9bc53 input=bdfb31598d4e47b9]*/
 {
     Py_ssize_t in, out;
     char ch;
@@ -1032,10 +1039,7 @@ binascii_a2b_qp_impl(PyObject *module, Py_buffer *data, int header)
             out++;
         }
     }
-    if ((rv = PyBytes_FromStringAndSize((char *)odata, out)) == NULL) {
-        PyMem_Free(odata);
-        return NULL;
-    }
+    rv = PyBytes_FromStringAndSize((char *)odata, out);
     PyMem_Free(odata);
     return rv;
 }
@@ -1059,9 +1063,9 @@ to_hex (unsigned char ch, unsigned char *s)
 binascii.b2a_qp
 
     data: Py_buffer
-    quotetabs: bool(accept={int}) = False
-    istext: bool(accept={int}) = True
-    header: bool(accept={int}) = False
+    quotetabs: bool = False
+    istext: bool = True
+    header: bool = False
 
 Encode a string using quoted-printable encoding.
 
@@ -1073,7 +1077,7 @@ are both encoded.  When quotetabs is set, space and tabs are encoded.
 static PyObject *
 binascii_b2a_qp_impl(PyObject *module, Py_buffer *data, int quotetabs,
                      int istext, int header)
-/*[clinic end generated code: output=e9884472ebb1a94c input=21fb7eea4a184ba6]*/
+/*[clinic end generated code: output=e9884472ebb1a94c input=e9102879afb0defd]*/
 {
     Py_ssize_t in, out;
     const unsigned char *databuf;
@@ -1240,10 +1244,7 @@ binascii_b2a_qp_impl(PyObject *module, Py_buffer *data, int quotetabs,
             }
         }
     }
-    if ((rv = PyBytes_FromStringAndSize((char *)odata, out)) == NULL) {
-        PyMem_Free(odata);
-        return NULL;
-    }
+    rv = PyBytes_FromStringAndSize((char *)odata, out);
     PyMem_Free(odata);
     return rv;
 }
@@ -1305,6 +1306,7 @@ binascii_exec(PyObject *module) {
 
 static PyModuleDef_Slot binascii_slots[] = {
     {Py_mod_exec, binascii_exec},
+    {Py_mod_multiple_interpreters, Py_MOD_PER_INTERPRETER_GIL_SUPPORTED},
     {0, NULL}
 };
 
