@@ -19,7 +19,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "btrace.h"
 #include "gdbthread.h"
 #include "inferior.h"
@@ -31,7 +30,7 @@
 #include "filenames.h"
 #include "regcache.h"
 #include "gdbsupport/rsp-low.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
 #include "cli/cli-utils.h"
 #include "extension.h"
 #include "gdbarch.h"
@@ -257,8 +256,8 @@ ftrace_new_function (struct btrace_thread_info *btinfo,
       insn_offset = prev->insn_offset + ftrace_call_num_insn (prev);
     }
 
-  btinfo->functions.emplace_back (mfun, fun, number, insn_offset, level);
-  return &btinfo->functions.back ();
+  return &btinfo->functions.emplace_back (mfun, fun, number, insn_offset,
+					  level);
 }
 
 /* Update the UP field of a function segment.  */
@@ -644,6 +643,18 @@ ftrace_update_function (struct btrace_thread_info *btinfo, CORE_ADDR pc)
 
 	    break;
 	  }
+
+	case BTRACE_INSN_AUX:
+	  /* An aux insn couldn't have switched the function.  But the
+	     segment might not have had a symbol name resolved yet, as events
+	     might not have an IP.  Use the current IP in that case and update
+	     the name.  */
+	  if (bfun->sym == nullptr && bfun->msym == nullptr)
+	    {
+	      bfun->sym = fun;
+	      bfun->msym = mfun;
+	    }
+	  return bfun;
 	}
     }
 
@@ -668,6 +679,11 @@ ftrace_update_insns (struct btrace_function *bfun, const btrace_insn &insn)
 {
   bfun->insn.push_back (insn);
 
+  if (insn.iclass == BTRACE_INSN_AUX)
+    bfun->flags |= BFUN_CONTAINS_AUX;
+  else
+    bfun->flags |= BFUN_CONTAINS_NON_AUX;
+
   if (record_debug > 1)
     ftrace_debug (bfun, "update insn");
 }
@@ -1060,11 +1076,10 @@ btrace_compute_ftrace_bts (struct thread_info *tp,
   switch_to_thread (tp);
 
   struct btrace_thread_info *btinfo;
-  struct gdbarch *gdbarch;
   unsigned int blk;
   int level;
 
-  gdbarch = target_gdbarch ();
+  gdbarch *gdbarch = current_inferior ()->arch ();
   btinfo = &tp->btrace;
   blk = btrace->blocks->size ();
 
@@ -1205,6 +1220,54 @@ pt_btrace_insn (const struct pt_insn &insn)
 	  pt_btrace_insn_flags (insn)};
 }
 
+#if defined (HAVE_PT_INSN_EVENT)
+/* Helper for events that will result in an aux_insn.  */
+
+static void
+handle_pt_aux_insn (btrace_thread_info *btinfo, btrace_function *bfun,
+		    std::string &aux_str, CORE_ADDR ip)
+{
+  btinfo->aux_data.emplace_back (std::move (aux_str));
+  bfun = ftrace_update_function (btinfo, ip);
+
+  btrace_insn insn {btinfo->aux_data.size () - 1, 0,
+		    BTRACE_INSN_AUX, 0};
+
+  ftrace_update_insns (bfun, insn);
+}
+
+/* Check if the last function segment contains real instructions, and not
+   only auxiliary instructions.  */
+
+static bool
+ftrace_last_bfun_contains_non_aux (btrace_thread_info *btinfo)
+{
+  if (btinfo->functions.empty ())
+    return false;
+
+  return ((btinfo->functions.back ().flags & BFUN_CONTAINS_NON_AUX) != 0);
+}
+#endif /* defined (HAVE_PT_INSN_EVENT) */
+
+#if (LIBIPT_VERSION >= 0x201)
+/* Translate an interrupt vector to a mnemonic string as defined for x86.
+   Returns nullptr if there is none.  */
+
+static const char *
+decode_interrupt_vector (const uint8_t vector)
+{
+  static const char *mnemonic[]
+    = { "#de", "#db", nullptr, "#bp", "#of", "#br", "#ud", "#nm",
+	"#df", "#mf", "#ts", "#np", "#ss", "#gp", "#pf", nullptr,
+	"#mf", "#ac", "#mc", "#xm", "#ve", "#cp" };
+
+  if (vector < (sizeof (mnemonic) / sizeof (mnemonic[0])))
+    return mnemonic[vector];
+
+  return nullptr;
+}
+#endif /* defined (LIBIPT_VERSION >= 0x201) */
+
 /* Handle instruction decode events (libipt-v2).  */
 
 static int
@@ -1218,6 +1281,7 @@ handle_pt_insn_events (struct btrace_thread_info *btinfo,
       struct btrace_function *bfun;
       struct pt_event event;
       uint64_t offset;
+      CORE_ADDR ip = 0;
 
       status = pt_insn_event (decoder, &event, sizeof (event));
       if (status < 0)
@@ -1232,9 +1296,18 @@ handle_pt_insn_events (struct btrace_thread_info *btinfo,
 	  if (event.status_update != 0)
 	    break;
 
-	  if (event.variant.enabled.resumed == 0 && !btinfo->functions.empty ())
+	  /* Only create a new gap if the last function segment contains
+	     non-aux instructions.  We could be at the beginning of the
+	     recording and could already have handled one or more events,
+	     like ptev_iret, that created aux insns.  In that case we don't
+	     want to create a gap or print a warning.  Note that if the last
+	     function segment contains only aux insns, we are guaranteed
+	     to be at the beginning of a recording or after a gap.  See
+	     handle_pt_aux_insn ().  */
+	  if (event.variant.enabled.resumed == 0
+	      && ftrace_last_bfun_contains_non_aux (btinfo))
 	    {
-	      bfun = ftrace_new_gap (btinfo, BDE_PT_DISABLED, gaps);
+	      bfun = ftrace_new_gap (btinfo, BDE_PT_NON_CONTIGUOUS, gaps);
 
 	      pt_insn_get_offset (decoder, &offset);
 
@@ -1256,9 +1329,7 @@ handle_pt_insn_events (struct btrace_thread_info *btinfo,
 #if defined (HAVE_STRUCT_PT_EVENT_VARIANT_PTWRITE)
 	case ptev_ptwrite:
 	  {
-	    uint64_t ip = 0;
-	    gdb::optional<std::string> ptw_string;
-	    btrace_insn_flags flags = 0;
+	    std::optional<std::string> ptw_string;
 
 	    /* Lookup the ip if available.  */
 	    if (event.ip_suppressed == 0)
@@ -1275,35 +1346,208 @@ handle_pt_insn_events (struct btrace_thread_info *btinfo,
 	    if (!ptw_string.has_value ())
 	      *ptw_string = hex_string (event.variant.ptwrite.payload);
 
-	    btinfo->aux_data.emplace_back (std::move (*ptw_string));
-
-	    if (!btinfo->functions.empty ()
-		&& !btinfo->functions.back ().insn.empty ())
-	      flags = btinfo->functions.back ().insn.back ().flags;
-
-	    /* Update insn list with ptw payload insn.  */
-	    struct btrace_insn ptw_insn;
-	    ptw_insn.aux_data_index = btinfo->aux_data.size () - 1;
-	    ptw_insn.size = 0;
-	    ptw_insn.iclass = BTRACE_INSN_AUX;
-	    ptw_insn.flags = flags;
-
-	    if (ip != 0)
-	      bfun = ftrace_update_function (btinfo, ip);
-	    else
-	      {
-		if (btinfo->functions.empty ())
-		  bfun = ftrace_new_function (btinfo, NULL, NULL);
-		else
-		  bfun = &btinfo->functions.back ();
-	      }
-
-	    bfun->flags |= BFUN_CONTAINS_AUX;
-	    ftrace_update_insns (bfun, ptw_insn);
+	    handle_pt_aux_insn (btinfo, bfun, *ptw_string, ip);
 
 	    break;
 	  }
 #endif /* defined (HAVE_STRUCT_PT_EVENT_VARIANT_PTWRITE) */
+
+#if (LIBIPT_VERSION >= 0x201)
+	case ptev_interrupt:
+	  {
+	    std::string aux_string = std::string (_("interrupt: vector = "))
+	      + hex_string (event.variant.interrupt.vector);
+
+	    const char *decoded
+	      = decode_interrupt_vector (event.variant.interrupt.vector);
+	    if (decoded != nullptr)
+	      aux_string += std::string (" (") + decoded + ")";
+
+	    if (event.variant.interrupt.has_cr2 != 0)
+	      {
+		aux_string += std::string (", cr2 = ")
+		  + hex_string (event.variant.interrupt.cr2);
+	      }
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.interrupt.ip;
+		aux_string += std::string (", ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_iret:
+	  {
+	    std::string aux_string = std::string (_("iret"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.iret.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_smi:
+	  {
+	    std::string aux_string = std::string (_("smi"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.smi.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_rsm:
+	  {
+	    std::string aux_string = std::string (_("rsm"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.rsm.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_sipi:
+	  {
+	    std::string aux_string = std::string (_("sipi: vector = "))
+	      + hex_string (event.variant.sipi.vector);
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_init:
+	  {
+	    std::string aux_string = std::string (_("init"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.init.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_vmentry:
+	  {
+	    std::string aux_string = std::string (_("vmentry"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.vmentry.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_vmexit:
+	  {
+	    std::string aux_string = std::string (_("vmexit"));
+
+	    if (event.variant.vmexit.has_vector != 0
+		|| event.variant.vmexit.has_vmxr != 0
+		|| event.variant.vmexit.has_vmxq != 0
+		|| event.ip_suppressed != 0)
+	      aux_string += std::string (":");
+
+	    if (event.variant.vmexit.has_vector != 0)
+	      {
+		aux_string += std::string (_(" vector = "))
+			      + hex_string (event.variant.vmexit.vector);
+
+		const char* decoded = decode_interrupt_vector
+					(event.variant.vmexit.vector);
+		if (decoded != nullptr)
+		  aux_string += std::string (" (") + decoded + ")";
+	      }
+
+	    if (event.variant.vmexit.has_vmxr != 0)
+	      {
+		std::string seperator = aux_string.back () == ':' ? "" : ",";
+		aux_string += seperator + std::string (" vmxr = ")
+			      + hex_string (event.variant.vmexit.vmxr);
+	      }
+
+	    if (event.variant.vmexit.has_vmxq != 0)
+	      {
+		std::string seperator = aux_string.back () == ':' ? "" : ",";
+		aux_string += seperator + std::string (" vmxq = ")
+			      + hex_string (event.variant.vmexit.vmxq);
+	      }
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.vmexit.ip;
+		std::string seperator = aux_string.back () == ':' ? "" : ",";
+		aux_string += seperator + std::string (" ip = ")
+			      + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_shutdown:
+	  {
+	    std::string aux_string = std::string (_("shutdown"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.shutdown.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_uintr:
+	  {
+	    std::string aux_string = std::string (_("uintr: vector = "))
+				     + hex_string (event.variant.uintr.vector);
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.uintr.ip;
+		aux_string += std::string (", ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+
+	case ptev_uiret:
+	  {
+	    std::string aux_string = std::string (_("uiret"));
+
+	    if (event.ip_suppressed == 0)
+	      {
+		ip = event.variant.uiret.ip;
+		aux_string += std::string (": ip = ") + hex_string (ip);
+	      }
+
+	    handle_pt_aux_insn (btinfo, bfun, aux_string, ip);
+	    break;
+	  }
+#endif /* defined (LIBIPT_VERSION >= 0x201) */
 	}
     }
 #endif /* defined (HAVE_PT_INSN_EVENT) */
@@ -1330,7 +1574,7 @@ handle_pt_insn_event_flags (struct btrace_thread_info *btinfo,
       struct btrace_function *bfun;
       uint64_t offset;
 
-      bfun = ftrace_new_gap (btinfo, BDE_PT_DISABLED, gaps);
+      bfun = ftrace_new_gap (btinfo, BDE_PT_NON_CONTIGUOUS, gaps);
 
       pt_insn_get_offset (decoder, &offset);
 
@@ -1937,8 +2181,8 @@ btrace_decode_error (enum btrace_format format, int errcode)
 	case BDE_PT_USER_QUIT:
 	  return _("trace decode cancelled");
 
-	case BDE_PT_DISABLED:
-	  return _("disabled");
+	case BDE_PT_NON_CONTIGUOUS:
+	  return _("non-contiguous");
 
 	case BDE_PT_OVERFLOW:
 	  return _("overflow");
@@ -2717,6 +2961,19 @@ pt_print_packet (const struct pt_packet *packet)
 		  packet->payload.ptw.ip ? (" ip") : (""));
       break;
 #endif /* defined (LIBIPT_VERSION >= 0x200)  */
+
+#if (LIBIPT_VERSION >= 0x201)
+    case ppt_cfe:
+      gdb_printf (("cfe %u: 0x%x%s"), packet->payload.cfe.type,
+		  packet->payload.cfe.vector,
+		  packet->payload.cfe.ip ? (" ip") : (""));
+      break;
+
+    case ppt_evd:
+      gdb_printf (("evd %u: 0x%" PRIx64 ""), packet->payload.evd.type,
+		  packet->payload.evd.payload);
+      break;
+#endif /* defined (LIBIPT_VERSION >= 0x201)  */
     }
 }
 
