@@ -15,7 +15,6 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "server.h"
 #include "target.h"
 #include "netbsd-low.h"
 #include "nat/netbsd-nat.h"
@@ -108,7 +107,7 @@ netbsd_process_target::post_create_inferior ()
 /* Implement the attach target_ops method.  */
 
 int
-netbsd_process_target::attach (unsigned long pid)
+netbsd_process_target::attach (int pid)
 {
   /* Unimplemented.  */
   return -1;
@@ -410,7 +409,7 @@ netbsd_process_target::wait (ptid_t ptid, struct target_waitstatus *ourstatus,
 	case TARGET_WAITKIND_THREAD_CREATED:
 	case TARGET_WAITKIND_THREAD_EXITED:
 	  /* The core needlessly stops on these events.  */
-	  /* FALLTHROUGH */
+	  [[fallthrough]];
 	case TARGET_WAITKIND_SPURIOUS:
 	  /* Spurious events are unhandled by the gdbserver core.  */
 	  if (ptrace (PT_CONTINUE, current_process ()->pid, (void *) 1, 0)
