@@ -1,4 +1,4 @@
-# Copyright (C) 2019-2023 Free Software Foundation, Inc.
+# Copyright (C) 2019-2024 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@ variables.
 import re
 import subprocess
 import platform
+from os import path
 import gdb
 
 class DebugLogger:
@@ -95,6 +96,9 @@ class DebugLogger:
 class IntelgtErrorReport():
     """Class to handle and highlight intelgt GPU debugging errors."""
 
+    """Eu debug path for KMD Linux driver"""
+    eu_debug_path = ""
+
     @staticmethod
     def exit_intelgt_session(
             input_err=None, exception=None, terminate=True):
@@ -131,12 +135,34 @@ Continuing with GPU-debugging disabled.
         """Run gdb.execute with to_string option set to True to capture the
         output from gdbserver.  In case of exception, highlight the error."""
         try:
-            gdb.execute(f"{command}", False, capture_output)
+            output = gdb.execute(f"{command}", False, capture_output)
+            # Do not suppress warnings, print them in YELLOW color.
+            if capture_output:
+                for line in output.splitlines():
+                    if line.startswith("warning:"):
+                        print(f"\033[93m{line}\033[0m")
             return True
         except gdb.error as gdb_exception:
             IntelgtErrorReport.exit_intelgt_session(exception=gdb_exception)
             return False
 
+    @staticmethod
+    def is_gdbserver_ze_installed():
+        """Checks if gdbserver-ze is installed and returns True if it does
+        otherwise returns False."""
+        try:
+            output = subprocess.check_output(
+                "which gdbserver-ze", shell=True).decode()
+        except (RuntimeError, subprocess.CalledProcessError):
+            output = ""
+
+        if ("gdbserver-ze" in output
+            or INTELGT_AUTO_ATTACH.gdbserver_install_path
+            or INTELGT_AUTO_ATTACH.gdbserver_path_env_var):
+            return True
+
+        return False
+
     @staticmethod
     def is_level_zero_debugging_enabled():
         """Gets environment variable 'ZET_ENABLE_PROGRAM_DEBUGGING' value and
@@ -144,11 +170,11 @@ Continuing with GPU-debugging disabled.
         env_var = IntelgtAutoAttach.get_env_variable(
             "ZET_ENABLE_PROGRAM_DEBUGGING", "0")
 
-        return env_var != "0"
+        return env_var == "1"
 
     @staticmethod
     def get_kmd_debug_enabled_status():
-        """ Return True if prelim_enable_eu_debug kernel settings is set to 1
+        """Return True if EU debug kernel settings is set to 1
         for all GPU devices otherwise it returns False."""
         try:
             """Check i915 prelim_enable_eu_debug settings."""
@@ -157,13 +183,18 @@ Continuing with GPU-debugging disabled.
                 return True
             if platform.system() != "Linux":
                 return False
-            find_out = subprocess.check_output(
-                "find /sys/devices | grep enable_eu_debug",
-                shell=True).decode()
-            lines = find_out.split('\n')
+
+            cmd = "find /sys/class/drm/card*/device/ -name 'enable_eudebug' \
+                    -o -name 'prelim_enable_eu_debug'"
+            find_out = subprocess.check_output(cmd, shell=True).decode()
+            lines = find_out.split("\n")
             if len(lines) == 0:
                 return False
 
+            # Set the eu_debug_path so that we can use it in
+            # is_initialization_error to distinguish between XeKMD and i915.
+            IntelgtErrorReport.eu_debug_path = re.sub(r"card\d+", "card*", lines[0])
+
             # All GPUs should have prelim_enable_eu_debug set to 1.
             for line in lines:
                 if line == "":
@@ -182,17 +213,68 @@ Continuing with GPU-debugging disabled.
         """Raise the error with appropriate reason if any pre-requisite is
         missing for the intelgt GPU debugging.  It is called before attempting
         to attach gdbserver-ze."""
+        if not IntelgtErrorReport.is_gdbserver_ze_installed():
+            IntelgtErrorReport.exit_intelgt_session("""
+GDBServer-ze executable is not found.  Add path to gdbserver-ze binary in
+INTELGT_AUTO_ATTACH_GDBSERVER_PATH or PATH.""")
+            return True
         if not IntelgtErrorReport.is_level_zero_debugging_enabled():
             IntelgtErrorReport.exit_intelgt_session("""
+Debugging of GPU offloaded code is not enabled.
 Set 'ZET_ENABLE_PROGRAM_DEBUGGING=1' in host process environment.""")
             return True
         if not IntelgtErrorReport.get_kmd_debug_enabled_status():
-            IntelgtErrorReport.exit_intelgt_session("""
-Set kernel parameter 'i915.enable_eu_debug=1' or set /sys/class/drm/card*/prelim_enable_eu_debug system files value to 1.""")
+            if "prelim_" in IntelgtErrorReport.eu_debug_path:
+                msg = """
+Set kernel parameter 'i915.debug_eu=1' or set /sys/class/drm/card*/prelim_enable_eu_debug system files value to 1."""
+            else:
+                msg = """
+Set /sys/class/drm/card*/device/enable_eudebug system files value to 1."""
+            IntelgtErrorReport.exit_intelgt_session(msg)
             return True
 
         return False
 
+class IntelgtHookBreakpoint(gdb.Breakpoint):
+    """Class to initialize IntelGT inferiors upon hit."""
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.used_once = False
+        self.silent = True
+
+    def stop(self):
+        """Called when the breakpoint is hit.  Initializes IntelGT inferiors
+        on the first hit and prevents further actions on subsequent hits.
+        Always returns False to ensure that the program does not stop at this
+        breakpoint.  Filter based on the library loaded at current PC.
+        This can be extended in the future to only start gdbserver-ze in the
+        context of GPU debugging."""
+        current_pc = gdb.selected_frame().pc()
+        solib_name = gdb.current_progspace().solib_name(current_pc)
+        if (not self.used_once and
+            solib_name in INTELGT_AUTO_ATTACH.loaded_objfiles):
+            self.used_once = True
+            INTELGT_AUTO_ATTACH.gt_inferior_init_pending = False
+            INTELGT_AUTO_ATTACH.init_gt_inferiors()
+        return False
+
+    def was_stopped(self):
+        """Return True if the breakpoint was already used, otherwise False."""
+        return self.used_once
+
+# pylint: disable-next=too-few-public-methods
+class IntelgtDeleteHook:
+    "Delete a hook breakpoint when the corresponding objfile goes away"
+    def __init__(self, brkpoint, filename):
+        self.brkpoint = brkpoint
+        self.filename = filename
+
+    def __call__(self, event):
+        if (self.brkpoint is not None and
+            self.filename == event.objfile.filename):
+            self.brkpoint.delete()
+            self.brkpoint = None
 
 # pylint: disable-next=too-many-instance-attributes
 class IntelgtAutoAttach:
@@ -204,17 +286,47 @@ class IntelgtAutoAttach:
         gdb.events.exited.connect(self.handle_exited_event)
         gdb.events.before_prompt.connect(self.handle_before_prompt_event)
         self.inf_dict = {}
+        self.loaded_objfiles = set()
         self.host_inf_for_auto_remove = None
-        self.is_nonstop = False
-        self.hook_bp = None
-        self.the_bp = None
         self.enable_schedule_multiple_at_gt_removal = False
+        self.gt_inferior_init_pending = False
         # Env variable to pass custom flags to gdbserver such as
         # "--debug" and "--remote-debug" for debugging purposes.
         self.gdbserver_args = self.get_env_variable(
             "INTELGT_AUTO_ATTACH_GDBSERVER_ARGS", "")
         if self.gdbserver_args != "":
             self.gdbserver_args = " " + self.gdbserver_args
+        # Allow a user-defined hook breakpoint in addition to the one
+        # defined by the level-zero IntelGT driver.
+        hook_locspec = self.get_env_variable("INTELGT_AUTO_ATTACH_HOOK")
+        if hook_locspec:
+            self.setup_hook_bp(hook_locspec)
+        self.gdbserver_path_env_var = None
+        self.gdbserver_install_path = None
+        self.set_gdbserver_ze_path()
+
+    def set_gdbserver_ze_path(self):
+        """Set path variables to the installed gdbserver-ze locations and
+        later gdbserver-ze installation takes precendence in the same order
+        just in case multiple binaries are available for the gdbserver-ze:
+        1) Check 'INTELGT_AUTO_ATTACH_GDBSERVER_PATH' in the inferior's
+        environment.
+        2) Check gdbserver-ze in the debugger installation path."""
+        if self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH") is not None:
+            print("""\033[31m
+INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH is deprecated. Use INTELGT_AUTO_ATTACH_GDBSERVER_PATH instead.
+\033[0m""")
+            self.gdbserver_path_env_var = \
+                    self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH")
+        self.gdbserver_path_env_var = \
+          self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_PATH")
+
+        auto_attach_path = path.dirname(__file__)
+        gdbserver_path = path.join(
+            auto_attach_path, "..", "..", "..", "..", "..", "..",
+                "debugger", "bin")
+        if path.exists(path.join(gdbserver_path, "gdbserver-ze")):
+            self.gdbserver_install_path = gdbserver_path
 
     @staticmethod
     def get_env_variable(var, default=None):
@@ -239,16 +351,11 @@ class IntelgtAutoAttach:
                 f"Skipping hook breakpoint for {event.new_objfile.filename} loaded event.")
             return
 
-        if not ('libze_loader.so' in event.new_objfile.filename or
-                'libze_intel_gpu.so' in event.new_objfile.filename or
-                'ze_loader.dll' in event.new_objfile.filename or
-                'ze_intel_gpu64.dll' in event.new_objfile.filename):
-            return
-
         if ('libze_intel_gpu.so' in event.new_objfile.filename or
             'ze_intel_gpu64.dll' in event.new_objfile.filename):
             DebugLogger.log(
                 f"received {event.new_objfile.filename} loaded event.")
+            self.loaded_objfiles.add(event.new_objfile.filename)
             if gdb.selected_inferior().was_attached:
                 # We just learnt about the library event and the inferior
                 # was attached.  It is possible that the level-zero backend
@@ -257,49 +364,31 @@ class IntelgtAutoAttach:
                 # initialization has not happened yet.  In that case we try
                 # again later via the hook BP.
                 DebugLogger.log("context might already be initialized.")
-                self.init_gt_inferiors()
+                self.gt_inferior_init_pending = True
             return
 
         if ('libze_loader.so' in event.new_objfile.filename or
             'ze_loader.dll' in event.new_objfile.filename):
-            self.the_bp = "zeContextCreate"
-        else:
-            return
+            DebugLogger.log(
+                f"received {event.new_objfile.filename} loaded event. "
+                "Setting up bp hook.")
+            hook = self.setup_hook_bp(f"-qualified zeContextCreate inferior "
+                                      f"{gdb.selected_inferior()}")
+            filename = event.new_objfile.filename
+            self.loaded_objfiles.add(filename)
+            gdb.events.free_objfile.connect(IntelgtDeleteHook(hook, filename))
 
-        DebugLogger.log(
-            f"received {event.new_objfile.filename} loaded event. "
-            "Setting up bp hook.")
-        self.setup_hook_bp()
+    def setup_hook_bp(self, locspec):
+        """Set a breakpoint at the location indicated by locspec."""
 
-    def setup_hook_bp(self):
-        """Set a breakpoint at the location indicated by self.the_bp.  If
-        not set, return and do nothing."""
-        if not self.the_bp:
-            DebugLogger.log("BP location for the hook is not set.")
-            return
-
-        self.hook_bp = gdb.Breakpoint(self.the_bp, type=gdb.BP_BREAKPOINT,
-                                      internal=1, temporary=0)
-        self.hook_bp.silent = True
-        commands = ("python gdb.function.intelgt_auto_attach" +
-                    ".INTELGT_AUTO_ATTACH.init_gt_inferiors()")
-        # Find out if we are in non-stop mode.  It is safe to do it
-        # here and only once, because the setting cannot be
-        # changed after the program starts executing.
-        nonstop_info = gdb.execute("show non-stop", False, True)
-        self.is_nonstop = nonstop_info.endswith("on.\n")
-
-        if self.is_nonstop:
-            command_suffix = "\ncontinue -a"
-        else:
-            command_suffix = "\ncontinue"
+        hook_bp = IntelgtHookBreakpoint(locspec, type=gdb.BP_BREAKPOINT,
+                                        internal=1, temporary=0)
 
         eclipse = self.get_env_variable("ECLIPSE")
         if eclipse is not None and eclipse.endswith("1"):
-            self.hook_bp.silent = False
-            command_suffix = ""
+            hook_bp.silent = False
 
-        self.hook_bp.commands = commands + command_suffix
+        return hook_bp
 
     @DebugLogger.log_call
     def remove_gt_inf_for(self, host_inf):
@@ -328,6 +417,16 @@ class IntelgtAutoAttach:
             if gt_inf.pid != 0:
                 gdb.execute(f"detach inferiors {gt_inf.num}", False, False)
 
+        # We need to explicitly make gdbserver-ze quit.
+        if gt_infs and gdb.selected_inferior() not in gt_infs:
+            cmd = f"inferior {gt_infs[0].num}"
+            self.protected_gdb_execute(cmd, True)
+
+        # Check if switching to gt inf was successful.
+        # Otherwise, we cannot send 'monitor exit' command.
+        if gdb.selected_inferior() in gt_infs:
+            gdb.execute("monitor exit")
+
         # Switch to host to be able to remove gt infs.
         # Catch exceptions, since the switch will likely raise
         # an error because the host inf also exited.
@@ -391,11 +490,16 @@ class IntelgtAutoAttach:
                 return True
             raise
 
+    @DebugLogger.log_call
     def handle_before_prompt_event(self):
         """Handler for GDB's 'before prompt' event.  Remove a gt inferior
         if such an inferior was stored for removal."""
         self.remove_gt_inf_if_stored_for_removal()
 
+        """If the host was attached, we init gt inferiors here."""
+        if self.gt_inferior_init_pending:
+            self.init_gt_inferiors()
+
     @DebugLogger.log_call
     def handle_exited_event(self, event):
         """Indication that an inferior's process has exited.
@@ -483,26 +587,20 @@ class IntelgtAutoAttach:
         if IntelgtErrorReport.is_initialization_error():
             return False
 
-        # Check 'INTELGT_AUTO_ATTACH_GDBSERVER_PATH' in the inferior's
-        # environment.
-        if self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH") is not None:
-            print("""\033[31m
-INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH is deprecated. Use INTELGT_AUTO_ATTACH_GDBSERVER_PATH instead.
-\033[0m""")
-            gdbserver_path_env_var = \
-                    self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH")
-        gdbserver_path_env_var = \
-          self.get_env_variable("INTELGT_AUTO_ATTACH_GDBSERVER_PATH")
-
         binary = "gdbserver-ze " + self.gdbserver_args
         if platform.system() == "Windows":
-            gdbserver_attach_str = f"{binary} --attach 127.0.0.1:PORT_PLACE_HOLDER {inf.pid}"
+            gdbserver_attach_str = f"{binary} --multi --once 127.0.0.1:PORT_PLACE_HOLDER"
         else:
-            gdbserver_attach_str = f"{binary} --attach - {inf.pid}"
+            gdbserver_attach_str = f"{binary} --multi --once -"
 
-        if gdbserver_path_env_var:
+        if self.gdbserver_path_env_var:
             gdbserver_attach_str = \
-              f"{gdbserver_path_env_var}/{gdbserver_attach_str}"
+              f"{self.gdbserver_path_env_var}/{gdbserver_attach_str}"
+        elif self.gdbserver_install_path:
+            gdbserver_attach_str = \
+              f"{self.gdbserver_install_path}/{gdbserver_attach_str}"
+
+        DebugLogger.log(f"gdbserver-ze attach comand: {gdbserver_attach_str}")
 
         try:
             self.make_native_gdbserver(inf, gdbserver_attach_str)
@@ -539,13 +637,36 @@ INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH is deprecated. Use INTELGT_AUTO_ATTACH_GDB
             print(f"intelgt: connecting to gdbserver-ze "
                   f"on port {gdbserver_port}.")
             IntelgtErrorReport.execute_with_errorcapture(
-                f"target remote :{gdbserver_port}", capture_output)
+                f"target extended-remote :{gdbserver_port}", capture_output)
         else:
             IntelgtErrorReport.execute_with_errorcapture(
-                f"target remote | {target_remote_str}", capture_output)
+                f"target extended-remote | {target_remote_str}", capture_output)
             print(f"intelgt: gdbserver-ze started for "
                   f"process {inf.pid}.")
 
+        # We execute the attach command in async mode, so that GDB
+        # will interrupt only one thread and resume it right after.
+        # This is much less intrusive and is fine to do because the
+        # host thread is blocked on the hook BP but will be resumed
+        # soon.
+        #
+        # However, if we are performing a live-attach in non-stop
+        # mode, we do not do an async attach.  In this case, we must
+        # be within an 'attach' command for the host inferior.
+        # Therefore, we do a synchronous attach to the device and let
+        # the user resume the threads.
+        #
+        # NOTE: If the user executed an 'attach &' for the host
+        # inferior, we would need to attach to the device and resume
+        # completely.  This case is not handled, yet.  The user still
+        # needs to resume the device explicitly.
+
+        async_mode = not self.gt_inferior_init_pending
+        attach_cmd = f"attach {inf.pid}" + (" &" if async_mode else "")
+        DebugLogger.log(f"command: {attach_cmd}")
+        gdb.execute(f"with print thread-events off -- {attach_cmd}",
+                    False, capture_output)
+
     @DebugLogger.log_call
     def init_gt_inferiors(self):
         """Create/initialize inferiors with gdbserver connection"""
@@ -567,10 +688,10 @@ INTELGT_AUTO_ATTACH_GDBSERVER_GT_PATH is deprecated. Use INTELGT_AUTO_ATTACH_GDB
 Connection name '{connection}' not recognized.""")
             return
 
-        self.hook_bp.delete()
-
         if host_inf not in self.inf_dict or self.inf_dict[host_inf] is None:
-            if not self.is_nonstop:
+            is_nonstop = gdb.execute("show non-stop",
+                                     to_string = True).endswith("on.\n")
+            if not is_nonstop:
                 gdb.execute("set schedule-multiple off")
             gdb.execute("add-inferior -hidden -no-connection", False, True)
 
@@ -585,6 +706,7 @@ Connection name '{connection}' not recognized.""")
                 # For the --attach scenario we use gt_inf; for the
                 # --multi scenario we use gt_inf's connection num.
                 self.inf_dict[host_inf] = (gt_inf, gt_inf.connection_num)
+
                 # Switch to the host inferior.
                 gdb.execute(f"inferior {host_inf.num}", False, True)
             # Fix ctrl-c while attaching to gt inferior.
@@ -596,15 +718,15 @@ Connection name '{connection}' not recognized.""")
                 # should be retried by setting up the hook bp.
                 if "attempting to attach too early" in str(ex):
                     self.handle_error(host_inf)
-                    self.setup_hook_bp()
                 else:
                     self.handle_error(host_inf, details=str(ex))
                     IntelgtErrorReport.exit_intelgt_session(exception=ex)
 
-            if not self.is_nonstop:
+            if not is_nonstop:
                 gdb.execute("set schedule-multiple on")
 
             self.set_suppress_notifications("on" if cli_suppressed else "off")
+            self.gt_inferior_init_pending = False
             return
 
     @staticmethod
