@@ -1,5 +1,6 @@
 /*
- * Copyright (c) 2013-2021, Intel Corporation
+ * Copyright (c) 2013-2024, Intel Corporation
+ * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
@@ -162,10 +163,12 @@ static struct ptunit_result create(struct section_fixture *sfix)
 	const char *name;
 	uint8_t bytes[] = { 0xcc, 0xcc, 0xcc, 0xcc, 0xcc };
 	uint64_t offset, size;
+	int errcode;
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	name = pt_section_filename(sfix->section);
@@ -182,8 +185,10 @@ static struct ptunit_result create(struct section_fixture *sfix)
 
 static struct ptunit_result create_bad_offset(struct section_fixture *sfix)
 {
-	sfix->section = pt_mk_section(sfix->name, 0x10ull, 0x0ull);
-	ptu_null(sfix->section);
+	int errcode;
+
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x10ull, 0x0ull);
+	ptu_int_eq(errcode, -pte_invalid);
 
 	return ptu_passed();
 }
@@ -193,10 +198,12 @@ static struct ptunit_result create_truncated(struct section_fixture *sfix)
 	const char *name;
 	uint8_t bytes[] = { 0xcc, 0xcc, 0xcc, 0xcc, 0xcc };
 	uint64_t offset, size;
+	int errcode;
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, UINT64_MAX);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, UINT64_MAX);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	name = pt_section_filename(sfix->section);
@@ -213,7 +220,10 @@ static struct ptunit_result create_truncated(struct section_fixture *sfix)
 
 static struct ptunit_result create_empty(struct section_fixture *sfix)
 {
-	sfix->section = pt_mk_section(sfix->name, 0x0ull, 0x10ull);
+	int errcode;
+
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x0ull, 0x10ull);
+	ptu_int_eq(errcode, -pte_invalid);
 	ptu_null(sfix->section);
 
 	return ptu_passed();
@@ -359,7 +369,8 @@ static struct ptunit_result get_overflow(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->ucount = UINT16_MAX;
@@ -380,7 +391,8 @@ static struct ptunit_result attach_overflow(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->acount = UINT16_MAX;
@@ -401,7 +413,8 @@ static struct ptunit_result attach_bad_ucount(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->acount = 2;
@@ -421,7 +434,8 @@ static struct ptunit_result map_change(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix_write(sfix, bytes);
@@ -439,7 +453,8 @@ static struct ptunit_result map_put(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -461,7 +476,8 @@ static struct ptunit_result unmap_nomap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_unmap(sfix->section);
@@ -477,7 +493,8 @@ static struct ptunit_result map_overflow(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->mcount = UINT16_MAX;
@@ -497,7 +514,8 @@ static struct ptunit_result get_put(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_get(sfix->section);
@@ -523,7 +541,8 @@ static struct ptunit_result attach_detach(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->ucount += 2;
@@ -553,7 +572,8 @@ static struct ptunit_result attach_bad_iscache(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	sfix->section->ucount += 2;
@@ -580,7 +600,8 @@ static struct ptunit_result detach_bad_iscache(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_attach(sfix->section, &iscache);
@@ -602,7 +623,8 @@ static struct ptunit_result map_unmap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -630,7 +652,8 @@ static struct ptunit_result attach_map(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_attach(sfix->section, &iscache);
@@ -666,7 +689,8 @@ static struct ptunit_result attach_bad_map(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_attach(sfix->section, &iscache);
@@ -691,7 +715,8 @@ static struct ptunit_result attach_map_overflow(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_attach(sfix->section, &iscache);
@@ -718,7 +743,8 @@ static struct ptunit_result read(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -744,7 +770,8 @@ static struct ptunit_result read_null(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -772,7 +799,8 @@ static struct ptunit_result read_offset(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -797,7 +825,8 @@ static struct ptunit_result read_truncated(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -821,7 +850,8 @@ static struct ptunit_result read_from_truncated(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x2ull, 0x10ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x2ull, 0x10ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -845,7 +875,8 @@ static struct ptunit_result read_nomem(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -868,7 +899,8 @@ static struct ptunit_result read_overflow(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -892,7 +924,8 @@ static struct ptunit_result read_overflow_32bit(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -916,7 +949,8 @@ static struct ptunit_result read_nomap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_read(sfix->section, buffer, 1, 0x0ull);
@@ -934,7 +968,8 @@ static struct ptunit_result read_unmap_map(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	status = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(status, 0);
 	ptu_ptr(sfix->section);
 
 	status = pt_section_map(sfix->section);
@@ -1073,7 +1108,8 @@ static struct ptunit_result stress(struct section_fixture *sfix,
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 #if defined(FEATURE_THREADS)
@@ -1099,7 +1135,8 @@ static struct ptunit_result init_no_bcache(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1122,7 +1159,8 @@ static struct ptunit_result bcache_alloc_free(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1151,7 +1189,8 @@ static struct ptunit_result bcache_alloc_twice(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1176,7 +1215,8 @@ static struct ptunit_result bcache_alloc_nomap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_alloc_bcache(sfix->section);
@@ -1193,7 +1233,8 @@ static struct ptunit_result memsize_nomap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_memsize(sfix->section, &memsize);
@@ -1211,7 +1252,8 @@ static struct ptunit_result memsize_unmap(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1235,7 +1277,8 @@ static struct ptunit_result memsize_map_nobcache(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1262,7 +1305,8 @@ static struct ptunit_result memsize_map_bcache(struct section_fixture *sfix)
 
 	sfix_write(sfix, bytes);
 
-	sfix->section = pt_mk_section(sfix->name, 0x1ull, 0x3ull);
+	errcode = pt_mk_section(&sfix->section, sfix->name, 0x1ull, 0x3ull);
+	ptu_int_eq(errcode, 0);
 	ptu_ptr(sfix->section);
 
 	errcode = pt_section_map(sfix->section);
@@ -1300,31 +1344,55 @@ static struct ptunit_result sfix_init(struct section_fixture *sfix)
 
 static struct ptunit_result sfix_fini(struct section_fixture *sfix)
 {
-	int thrd;
+	char *filename;
+	FILE *file;
+	int thrd, errcode;
 
 	ptu_test(ptunit_thrd_fini, &sfix->thrd);
 
-	for (thrd = 0; thrd < sfix->thrd.nthreads; ++thrd)
-		ptu_int_eq(sfix->thrd.result[thrd], 0);
-
 	if (sfix->section) {
 		pt_section_put(sfix->section);
 		sfix->section = NULL;
 	}
 
-	if (sfix->file) {
-		fclose(sfix->file);
-		sfix->file = NULL;
+	filename = sfix->name;
+	file = sfix->file;
+	sfix->name = NULL;
+	sfix->file = NULL;
 
-		if (sfix->name)
-			remove(sfix->name);
+	/* Try removing the file while we still have it open to avoid races
+	 * with others re-using the temporary filename.
+	 *
+	 * On some systems that may not be possible and we can choose between:
+	 *
+	 *   - guaranteed leaking files or
+	 *   - running the risk of removing someone elses file
+	 *
+	 * We choose the latter.  Assuming those systems behave consistently,
+	 * removing someone elses file should only succeed if it isn't open at
+	 * the moment we try removing it.  Given that this is a temporary file,
+	 * we should be able to rule out accidental name clashes with
+	 * non-termporary files.
+	 */
+	if (filename && file) {
+		errcode = remove(filename);
+		if (!errcode) {
+			free(filename);
+			filename = NULL;
+		}
 	}
 
-	if (sfix->name) {
-		free(sfix->name);
-		sfix->name = NULL;
+	if (file)
+		fclose(file);
+
+	if (filename) {
+		(void) remove(filename);
+		free(filename);
 	}
 
+	for (thrd = 0; thrd < sfix->thrd.nthreads; ++thrd)
+		ptu_int_eq(sfix->thrd.result[thrd], 0);
+
 	return ptu_passed();
 }
 
